// This macro rotates the uCT image and the MRI image to reflect the actual positioning of the sample
// during MRI scanning (sagittal, cartilage tissue upwards)
//
// The uCT scan is rotated by 90 degrees counter-clockwise and flipped horizontally
// The MRI image is rotated by 90 degrees clockwise
//
// (c) 2022 Sinisa Skokic, Laboratory for Regenerative Neuroscience, University of Zagreb School of Medicine
// contact: <name>.<surname>@hiim.hr


function leftPad(n, width) {
	// serves to add leading zeros to ROI numbers when renaming
	s =""+n;
  	while (lengthOf(s)<width)
   	   s = "0"+s;
  	return s;
}

// clear ROI manager
if (roiManager("count") > 0) {
	roiManager("deselect");
	roiManager("delete");
}

// select input file/folder (CTAn sagittal images)
// set output folder
  path_CT = File.openDialog("Load CTAn sagittal images: open folder and select first image in the stack");
  dir_CT = File.getParent(path_CT);
  //print("root folder = "+dir_root_of_CTroi);

  saveDir =   getDirectory("select destination folder for output files.");

// load input files (uCT images) and form a stack
  fileName_CT = File.getName(path_CT);
  fileName_CTscaled = "uCT STACK scaled to MRI resolution";

  run("Image Sequence...", "open=["+dir_CT+"/"+fileName_CT+"] file=.bmp sort");
  image_uCT_stack = getImageID();
  run("Scale...", "x=0.2 y=0.2 z=1.0 depth="+nSlices+" interpolation=Bilinear process create");
  run("Rotate 90 Degrees Left");
  run("Flip Horizontally", "stack");
  run("Make Substack...", "  slices=3-"+nSlices+"-20");  // keep every 20th slice (slice thickness ratio CT/MRI)
  image_uCT_scaled_stack = getImageID();
  saveAs("tiff", saveDir+fileName_CTscaled);

  centralSlice = round(nSlices/2);
  setSlice(centralSlice);
  run("Duplicate...", " ");
  image_uCT_scaled_slice = getImageID();
  fileName_CTalign = fileName_CTscaled+" slice "+leftPad(centralSlice, 2);
  saveAs("tiff", saveDir+"/"+fileName_CTalign);

  selectImage(image_uCT_stack); close();
  selectImage(image_uCT_scaled_stack); close();
  


 
// load original PDw MRI image and pick central slice for alignment
  path_PDw = File.openDialog("Load PDw image");
  dir_PDw = File.getParent(path_PDw);
  fileName_PDw = File.getName(path_PDw);
  open(path_PDw); // open PDw file
  run("Rotate 90 Degrees Right");
  image_PDw = getImageID();
  title_PDw = getTitle();

  selectImage(image_uCT_scaled_slice);
  
  waitForUser("find MRI PDw slice most similar to uCT slice n. "+centralSlice+", then click OK.");
  selectImage(image_PDw); 
  currentSlice = getSliceNumber();
  run("Duplicate...", " ");
  image_PDw_forAlign = getImageID();
  fileName_PDw_forAlign = substring(fileName_PDw, 0, lengthOf(fileName_PDw)-4) + " slice "+leftPad(currentSlice,2);
  saveAs("tiff", saveDir+"/"+fileName_PDw_forAlign);

  selectImage(image_PDw); close();
  selectImage(image_PDw_forAlign); close();


  // check and finalize alignment in external macro "Align_images"
  waitForUser("run macro Align_images with central slice of the original MRI PDw image.\n"+
  	"NOTE: scaled uCT scans are saved in folder "+saveDir+"\n"+
  	"After you are done, mark down translation in X and Y and click OK.");


// load uCT ROIset, adapt it to MRI resolution and correct its position with 
// translation parameters verified in external macro "Align_images"
dialog_title = "ROI position correction";
Dialog.create(dialog_title); 
message1 = "If CT and MRI images are not perfectly aligned, enter the values of displacement\n"+
	"measured with Align_Images tool.";
message2 = "If no correction is needed, leave default values and click on OK.";
Dialog.addMessage(message1);
Dialog.addMessage(message2);
Dialog.addNumber("Translation in X [px]", 0);
Dialog.addNumber("Translation in Y [px]", 0);

Dialog.show();

trX = Dialog.getNumber();
trY = Dialog.getNumber();  
  
// select input file/folder (CTAn sagittal ROI images)
  path_CTroi = File.openDialog("Load CTAn sagittal ROI images: open folder and select first image in the stack");
  dir_CTroi = File.getParent(path_CTroi);
  dir_root_of_CTroi = File.getParent(dir_CTroi);
  //print("root folder = "+dir_root_of_CTroi);
  fileName_CTroi = File.getName(path_CTroi);
  run("Image Sequence...", "open=["+dir_CTroi+"/"+fileName_CTroi+"] file=.bmp sort");
  run("Scale...", "x=0.2 y=0.2 z=1.0 depth="+nSlices+" interpolation=Bilinear process create");
  run("Rotate 90 Degrees Left");
  run("Flip Horizontally", "stack");
  run("Make Substack...", "  slices=3-"+nSlices+"-20");  // keep every 20th slice (slice thickness ratio CT/MRI)
  run("Make Binary", "method=Default background=Dark calculate black");

for (i = 1; i <= nSlices; i++) {
    setSlice(i);
    run("Create Selection");
    if (is("area") == 1) {
    	run("Interpolate", "interval=0.5 smooth");  // smoothen new ROIs
	   	roiManager("Add");
    }
}

// save original ROIs created from uCT VOI folder
fileName_ROIset = "uCT ROIset adapted for MRI.zip";
roiManager("deselect");
roiManager("save selected", saveDir+fileName_ROIset);


// update each ROI position with entered translation parameters
// and save the updated ROI compatible with MRI PDw scan.
if (trX != 0 || trY != 0) {
	for (i = 0; i < roiManager("count"); i++) {
	    roiManager("select", i);
	    Roi.getBounds(x, y, width, height);
	    Roi.move(x+trX, y+trY);
	    roiManager("update");
    }
    // save created ROIs to a different file
	fileName_ROIset_corr = "uCT ROIset adapted for MRI - correct position.zip";
	roiManager("deselect");
	roiManager("save selected", saveDir+fileName_ROIset_corr);
}


close("\\Others");
rename("CT ROIset adapted for MRI - binary");




