// This macro calculates the percentage of bone (dark pixels) within the given ROI set
// based on the loaded MRI images without fat suppression (PDw) and with fat suppression (PDwFS).
//
// The formula used is given in the paper Duvancic et al. "Novel micro-MRI approach for subchondral 
// trabecular bone analysis in patients with hip osteoarthritis is comparable to micro-CT" (CMJ-5251).
//
// Outuput summary (average value, deviation) are computed using the Combined Variance formulae 
// in order to get true voxel-wise averages (as opposed to slice-wise averages, which do not take into account
// the sizes of individual ROIs in different slices). 
//
// NOTES: 
// 1) program assumes that ROIset was created on the ROTATED IMAGE (90 deg clockwise)
//    in which the cartilage is on top of the sample, as it really was in the MRI scanner (sagittal slice orientation)
// 2) for correct visualisation, a customised lookup table (LUT) is necessary.
//    The LUT is based on FIJI/ImageJ LUT "physics", with black color assigned to the lowest gray value assigned
//    In this macro, the new custom LUT is called "physics_blackZero".
//    The customised LUT should be copied to the corresponding FIJI/ImageJ subfolder.
//
// (c) 2022 Sinisa Skokic, Laboratory for Regenerative Neuroscience, University of Zagreb School of Medicine
// contact: <name>.<surname>@hiim.hr


function leftPad(n, width) {
	// serves to add leading zeros to ROI numbers when renaming
	s =""+n;
  	while (lengthOf(s)<width)
   	   s = "0"+s;
  	return s;
}

black_treshold = 50;
black_treshold_FS = 50;
setForegroundColor(255, 255, 255); setBackgroundColor(0, 0, 0);


run("Clear Results");   // clear previous results in Results table
print("\\Clear");   // clear log window
if (roiManager("count") > 0) {
	roiManager("deselect"); roiManager("delete");
}
run("Select None");


// load input files
  path_PDw = File.openDialog("Load PDw image");
  dir_PDw = File.getParent(path_PDw);
  fileName_PDw = File.getName(path_PDw);
  
  path_PDwFS = File.openDialog("Load PDw-FS image");
  dir_PDwFS = File.getParent(path_PDwFS);
  fileName_PDwFS = File.getName(path_PDwFS);

  path_ROI = File.openDialog("Load ROIset");
  dir_ROI = File.getParent(path_ROI);
  fileName_ROI = File.getName(path_ROI);

  saveDir =   getDirectory("select destination folder for output files.");

  // get image info for PDw and PDw-FS files
  open(path_PDw); // open PDw file
  image_PDw = getImageID();
  title_PDw = getTitle();
  //
  open(path_PDwFS); // open the T2map file
  image_PDwFS = getImageID();
  title_PDwFS = getTitle();

  //
  open(path_ROI); // open the ROIset file
  nROI = roiManager("count");
  //
  getVoxelSize(px_w, px_h, px_d, px_unit);
  wdth = getValue("Width")/px_w;
  hght = getValue("Height")/px_h;
  print("PDw image width = "+wdth+"; image height = "+hght+"; voxel size [mm]= "+px_w+" / "+px_h+" / "+px_d);
  if (hght > wdth) {
  	selectImage(image_PDw);
  	run("Rotate 90 Degrees Right");
  	run("Duplicate...", "duplicate");  // create a spare copy for FAT/WATER ratio calculation
  	image_PDw_copy = getImageID();
  	title_PDw_copy = title_PDw + " with F/W ratio overlay";
  	rename(title_PDw_copy);
  	selectImage(image_PDwFS);
  	run("Rotate 90 Degrees Right");
  	run("Duplicate...", "duplicate");  // create a spare copy for FAT/WATER ratio calculation
  	image_PDwFS_copy = getImageID();
  	title_PDwFS_copy = getTitle();
  	print("Images vertically oriented, should be horizontally. Rotated by 90deg.");
  }

//////
// keep only bone pixels in PDw and PDw-FS images
selectImage(image_PDw); 
setSlice(1); setPixel(0, 0, 65535); setPixel(getWidth(),getHeight(),0); // bug fix: add 1 white and 1 black pixel and
run("Enhance Contrast...", "saturated=0.01 normalize process_all use"); // normalize stack histogram before converting to 8-bit
run("8-bit"); 															// to ensure stable 8-bit conversion 
																		// which doesn't depend on the content of
																		// the first slice in the stack

for(i = 0; i<roiManager("count"); i++) {
	roiManager("select", i);
	run("Macro...", "code=[v=255*(v<"+black_treshold+")]");  // keep only bone pixels
	run("Make Inverse"); run("Clear", "slice"); // clear everything outside ROI
}
roiManager("deselect"); run("Select None");


//
selectImage(image_PDwFS); 
setSlice(1); setPixel(0, 0, 65535); setPixel(getWidth(),getHeight(),0); // bug fix: add 1 white and 1 black pixel and
run("Enhance Contrast...", "saturated=0.01 normalize process_all use"); // normalize stack histogram before converting to 8-bit
run("8-bit");

for(i = 0; i<nROI; i++) {
	roiManager("select", i);
	run("Macro...", "code=[v=255*(v<"+black_treshold_FS+")]");  // keep only bone pixels
	run("Make Inverse"); run("Clear", "slice"); // clear everything outside ROI
}
roiManager("deselect"); run("Select None");


// multiply filtered PDw and PDw-FS images (= keep BONE pixels which are detected as bone in both PDw and PDw-FS)
imageCalculator("Multiply create stack", image_PDw, image_PDwFS);
image_bone = getImageID();
title_bone = "BONE - calculated mask";
rename(title_bone);
print(saveDir+title_bone+".nii"); 
run("NIfTI-1", "save=["+saveDir+title_bone+".nii]");   // save Fat/Water ratio image as Nifti
waitForUser("provjeri BONE-CALCULATED sliku nakon filtriranja");

// FAT-WATER MASK - inverse of bone mask within selected ROI
run("Select None");
run("Duplicate...", "duplicate");
image_fat = getImageID();
title_fat = "FAT-WATER - calculated mask";
rename(title_fat);
selectImage(image_fat);
for(i = 0; i<nROI; i++) {
	roiManager("select", i);
	run("Invert", "slice");
}
print(saveDir+title_fat+".nii");
run("NIfTI-1", "save=["+saveDir+title_fat+".nii]");    // save Fat-Water pixel mask as Nifti
ROIindexes = newArray(nROI);
for(i = 0; i<nROI; i++) {
	roiManager("select", i); 
	sl = getSliceNumber();
	ROIindexes[i] = i;
	run("Select None");
	run("Options...", "iterations=1 count=1 black");  // make sure bright pixels are selected in "Create Selection"
	run("Create Selection"); Roi.setPosition(0,sl,0);
	roiManager("add"); roiManager("select", roiManager("count")-1);
	roiManager("rename", "FAT-WATER mask - slice "+leftPad(sl, 2));
}
roiManager("deselect"); run("Select None");
waitForUser("check FAT-WATER MASK image after processing");


// measure BONE VOLUME within selected ROIset
selectImage(image_bone);
run("Set Measurements...", "area area_fraction stack redirect=None decimal=4");
roiManager("select", ROIindexes);
roiManager("Measure");
boneVOLUME = 0; totalVOLUME = 0;
for (br=0; br<nResults; br++) {
	setResult("Bone Area",br,getResult("Area", br)*getResult("%Area", br)/100);  // add calculated volume to the table
	totalVOLUME = totalVOLUME + getResult("Area", br)*px_d;
	boneVOLUME = boneVOLUME + getResult("Bone Area", br)*px_d;
}
setResult("Label", nResults, "TOTAL VOLUME [mm3]");  // add new row
setResult("Area", nResults -1, totalVOLUME);   // write in the current last row
setResult("Bone Area", nResults -1, boneVOLUME);  
setResult("Label", nResults, "TOTAL FRACTION");  // add new row
setResult("%Area", nResults -1, 100*boneVOLUME/totalVOLUME);   // write in the current last row
waitForUser("Save measured volumetry");
saveAs("Results");


///// FAT-WATER RATIO CALCULATION 
//
// uses copies of PDw and PDw-FS images stored in original 16-bit form, transformed to 32-bit for correct calculation
imageCalculator("Subtract create 32-bit stack", title_PDw_copy, title_PDwFS_copy);
image_fatOnly = getImageID();
title_fatOnly = "fatOnly TMP";
selectImage(image_fatOnly);
rename(title_fatOnly);
imageCalculator("Divide create 32-bit stack", image_fatOnly, image_PDw_copy);
image_FWratio = getImageID();
title_FWratio = "FAT-WATER ratio";
selectImage(image_FWratio);
rename(title_FWratio);
roiManager("deselect"); run("Select None");
run("NIfTI-1", "save=["+saveDir+title_FWratio+".nii]");   // save Fat/Water ratio image as Nifti
//
// use FAT/WATER mask to measure average FAT/WATER ratio in ROI
// delete all not-fat pixels and all pixels outside ROI
ROIindexes = newArray(nROI);
selectImage(image_FWratio); 
setMinAndMax(0, 1); run("physics_blackZero");  // set colormap and range (blue = 0% fat, red = 100% fat)
selectImage(image_PDw_copy); 
setSlice(1); setPixel(0, 0, 65535); setPixel(getWidth(),getHeight(),0); // bug fix: add 1 white and 1 black pixel and
run("Enhance Contrast...", "saturated=0.1 normalize process_all use"); // normalize stack histogram before converting to 8-bit
run("RGB Color");  // convert to RGB image to be able to display colored overlay
for(i = 0; i<nROI; i++) {
	selectImage(image_FWratio); roiManager("select", nROI + i);
	run("Make Inverse"); run("Clear", "slice"); // clear everything outside ROI
	ROIindexes[i] = nROI + i;	run("Select None");
	selectImage(image_PDw_copy); roiManager("select", nROI + i); run("Select None");  // jump to same slice
	run("Add Image...", "image=[FAT-WATER ratio] x=0 y=0 opacity=100 zero");
}
// measure Fat/water ratio
run("Clear Results");   // clear previous results in Results table
selectImage(image_FWratio); 
roiManager("select", ROIindexes);
run("Set Measurements...", "area mean standard stack redirect=None decimal=4");
roiManager("Measure");
// save FAT-WATER ROIs
fileName_ROIset_FW = substring(fileName_ROI, 0, lengthOf(fileName_ROI)-4) + "_FatWaterAnalysis.zip";
roiManager("save selected", saveDir+fileName_ROIset_FW);
// to correctly calculate average F/W ratio and variance in the whole sample
// combined variance and mean must be calculated from measured results
// to take into account different ROI size in each slice
// REF: MathZone / Combined Variance
ROIarea = newArray(nResults);
ROImean = newArray(nResults);
ROIdev = newArray(nResults);
ROIvariance = newArray(nResults);
comb_mean = 0; comb_dev = 0; comb_var = 0; total_area = 0;
for (i=0; i < nResults; i++) {
	ROImean[i] = getResult("Mean", i);
	ROIarea[i] = getResult("Area", i);
	ROIdev[i] = getResult("StdDev", i);
	ROIvariance[i] = ROIdev[i]*ROIdev[i];
	comb_mean = comb_mean + ROIarea[i]*ROImean[i];
	total_area = total_area + ROIarea[i];
}
comb_mean = comb_mean/total_area;
for (i=0; i < nResults; i++) {
	comb_var = comb_var + ROIarea[i]*(ROIvariance[i] + Math.sqr(ROImean[i] - comb_mean));
	print("i = "+i+" | area = "+ROIarea[i]+", mean = "+ROImean[i]+", dev = "+ROIdev[i]+", var = "+ROIvariance[i]+
		" | comb.mean = "+comb_mean+" | comb.var.brojnik = "+comb_var);
}
comb_var = comb_var/total_area;
comb_dev = sqrt(comb_var);
// add average to measured results and save
setResult("Label", nResults, "AVERAGE");  // add new row
setResult("Mean", nResults -1, comb_mean);   // write in the current last row
setResult("StdDev", nResults -1, comb_dev);   // write in the current last row
waitForUser("Save FAT-WATER ratio measurements");
saveAs("Results");

selectImage(image_PDw_copy); 
fileName_PDw_copy = "PDw with Fat-Water overlay";
saveAs("tiff", saveDir+fileName_PDw_copy);
close("\\Others");



